@@ -12,10 +12,10 @@ use serde::de::{Deserialize, Deserializer, MapAccess, SeqAccess, Visitor};
 
 use toml;
 
-use bindgen::ir::annotation::AnnotationSet;
-use bindgen::ir::path::Path;
-use bindgen::ir::repr::ReprAlign;
-pub use bindgen::rename::RenameRule;
+use crate::bindgen::ir::annotation::AnnotationSet;
+use crate::bindgen::ir::path::Path;
+use crate::bindgen::ir::repr::ReprAlign;
+pub use crate::bindgen::rename::RenameRule;
 
 pub const VERSION: &str = env!("CARGO_PKG_VERSION");
 
@@ -201,6 +201,28 @@ impl FromStr for ItemType {
 
 deserialize_enum_str!(ItemType);
 
+/// Type which specifies the sort order of functions
+#[derive(Debug, Clone, PartialEq)]
+pub enum SortKey {
+    Name,
+    None,
+}
+
+impl FromStr for SortKey {
+    type Err = String;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        use self::SortKey::*;
+        Ok(match &*s.to_lowercase() {
+            "name" => Name,
+            "none" => None,
+            _ => return Err(format!("Unrecognized sort option: '{}'.", s)),
+        })
+    }
+}
+
+deserialize_enum_str!(SortKey);
+
 /// Settings to apply when exporting items.
 #[derive(Debug, Clone, Deserialize, Default)]
 #[serde(rename_all = "snake_case")]
@@ -214,6 +236,8 @@ pub struct ExportConfig {
     pub exclude: Vec<String>,
     /// Table of name conversions to apply to item names
     pub rename: HashMap<String, String>,
+    /// Table of raw strings to prepend to the body of items.
+    pub pre_body: HashMap<String, String>,
     /// Table of raw strings to append to the body of items.
     pub body: HashMap<String, String>,
     /// A prefix to add before the name of every item
@@ -229,7 +253,11 @@ impl ExportConfig {
         self.item_types.is_empty() || self.item_types.contains(&item_type)
     }
 
-    pub(crate) fn extra_body(&self, path: &Path) -> Option<&str> {
+    pub(crate) fn pre_body(&self, path: &Path) -> Option<&str> {
+        self.pre_body.get(path.name()).map(|s| s.trim_matches('\n'))
+    }
+
+    pub(crate) fn post_body(&self, path: &Path) -> Option<&str> {
         self.body.get(path.name()).map(|s| s.trim_matches('\n'))
     }
 
@@ -279,12 +307,16 @@ pub struct FunctionConfig {
     pub prefix: Option<String>,
     /// Optional text to output after each function declaration
     pub postfix: Option<String>,
-    /// The way to annotation this function as #[must_use].
+    /// The way to annotation this function as #[must_use]
     pub must_use: Option<String>,
     /// The style to layout the args
     pub args: Layout,
     /// The rename rule to apply to function args
     pub rename_args: Option<RenameRule>,
+    /// An optional macro to use when generating Swift function name attributes
+    pub swift_name_macro: Option<String>,
+    /// Sort key for function names
+    pub sort_by: SortKey,
 }
 
 impl Default for FunctionConfig {
@@ -295,6 +327,8 @@ impl Default for FunctionConfig {
             must_use: None,
             args: Layout::Auto,
             rename_args: None,
+            swift_name_macro: None,
+            sort_by: SortKey::Name,
         }
     }
 }
@@ -391,7 +425,7 @@ impl StructConfig {
 }
 
 /// Settings to apply to generated enums.
-#[derive(Debug, Clone, Default, Deserialize)]
+#[derive(Debug, Clone, Deserialize)]
 #[serde(rename_all = "snake_case")]
 #[serde(deny_unknown_fields)]
 #[serde(default)]
@@ -425,11 +459,34 @@ pub struct EnumConfig {
     /// This is only generated if a copy constructor for the same tagged enum is
     /// generated as well.
     pub derive_tagged_enum_copy_assignment: bool,
+    /// Declare the enum as an enum class.
+    /// Only relevant when targeting C++.
+    pub enum_class: bool,
     /// Whether to generate empty, private default-constructors for tagged
     /// enums.
     pub private_default_tagged_enum_constructor: bool,
 }
 
+impl Default for EnumConfig {
+    fn default() -> EnumConfig {
+        EnumConfig {
+            rename_variants: None,
+            add_sentinel: false,
+            prefix_with_name: false,
+            derive_helper_methods: false,
+            derive_const_casts: false,
+            derive_mut_casts: false,
+            cast_assert_name: None,
+            must_use: None,
+            derive_tagged_enum_destructor: false,
+            derive_tagged_enum_copy_constructor: false,
+            derive_tagged_enum_copy_assignment: false,
+            enum_class: true,
+            private_default_tagged_enum_constructor: false,
+        }
+    }
+}
+
 impl EnumConfig {
     pub(crate) fn add_sentinel(&self, annotations: &AnnotationSet) -> bool {
         if let Some(x) = annotations.bool("add-sentinel") {
@@ -473,6 +530,12 @@ impl EnumConfig {
         }
         self.derive_tagged_enum_copy_assignment
     }
+    pub(crate) fn enum_class(&self, annotations: &AnnotationSet) -> bool {
+        if let Some(x) = annotations.bool("enum-class") {
+            return x;
+        }
+        self.enum_class
+    }
     pub(crate) fn private_default_tagged_enum_constructor(
         &self,
         annotations: &AnnotationSet,
