@@ -7,8 +7,9 @@ use std::io::Write;
 
 use syn;
 
-use bindgen::config::Config;
-use bindgen::writer::SourceWriter;
+use crate::bindgen::cargo::cargo_metadata::Dependency;
+use crate::bindgen::config::Config;
+use crate::bindgen::writer::SourceWriter;
 
 #[derive(PartialEq, Eq)]
 enum DefineKey<'a> {
@@ -19,29 +20,27 @@ enum DefineKey<'a> {
 impl<'a> DefineKey<'a> {
     fn load(key: &str) -> DefineKey {
         // TODO: dirty parser
-        if key.contains("=") {
-            let mut splits = key.trim().split("=");
-
-            let name = if let Some(name) = splits.next() {
-                name.trim()
-            } else {
-                return DefineKey::Boolean(key);
-            };
-
-            let value = if let Some(value) = splits.next() {
-                value.trim()
-            } else {
-                return DefineKey::Boolean(key);
-            };
-
-            if splits.next().is_some() {
-                return DefineKey::Boolean(key);
-            }
-
-            DefineKey::Named(name, value)
-        } else {
+        if !key.contains('=') {
             return DefineKey::Boolean(key);
         }
+
+        let mut splits = key.trim().split('=');
+
+        let name = match splits.next() {
+            Some(n) => n.trim(),
+            None => return DefineKey::Boolean(key),
+        };
+
+        let value = match splits.next() {
+            Some(v) => v.trim(),
+            None => return DefineKey::Boolean(key),
+        };
+
+        if splits.next().is_some() {
+            return DefineKey::Boolean(key);
+        }
+
+        DefineKey::Named(name, value)
     }
 }
 
@@ -86,7 +85,7 @@ impl fmt::Display for Cfg {
 
 impl Cfg {
     pub fn join(cfgs: &[Cfg]) -> Option<Cfg> {
-        if cfgs.len() == 0 {
+        if cfgs.is_empty() {
             None
         } else {
             Some(Cfg::All(cfgs.to_owned()))
@@ -106,17 +105,14 @@ impl Cfg {
         let mut configs = Vec::new();
 
         for attr in attrs {
-            match attr.interpret_meta() {
-                Some(syn::Meta::List(syn::MetaList { ident, nested, .. })) => {
-                    if ident != "cfg" || nested.len() != 1 {
-                        continue;
-                    }
+            if let Ok(syn::Meta::List(syn::MetaList { path, nested, .. })) = attr.parse_meta() {
+                if !path.is_ident("cfg") || nested.len() != 1 {
+                    continue;
+                }
 
-                    if let Some(config) = Cfg::load_single(nested.first().unwrap().value()) {
-                        configs.push(config);
-                    }
+                if let Some(config) = Cfg::load_single(nested.first().unwrap()) {
+                    configs.push(config);
                 }
-                _ => {}
             }
         }
 
@@ -127,63 +123,70 @@ impl Cfg {
         }
     }
 
+    pub fn load_metadata(dependency: &Dependency) -> Option<Cfg> {
+        dependency
+            .target
+            .as_ref()
+            .map(|target| {
+                syn::parse_str::<syn::Meta>(target)
+                    .expect("error parsing dependency's target metadata")
+            })
+            .and_then(|target| {
+                if let syn::Meta::List(syn::MetaList { path, nested, .. }) = target {
+                    if !path.is_ident("cfg") || nested.len() != 1 {
+                        return None;
+                    }
+                    Cfg::load_single(nested.first().unwrap())
+                } else {
+                    None
+                }
+            })
+    }
+
     fn load_single(item: &syn::NestedMeta) -> Option<Cfg> {
-        match item {
-            &syn::NestedMeta::Meta(syn::Meta::Word(ref ident)) => {
-                Some(Cfg::Boolean(format!("{}", ident)))
+        Some(match *item {
+            syn::NestedMeta::Meta(syn::Meta::Path(ref path)) => {
+                Cfg::Boolean(format!("{}", path.segments.first().unwrap().ident))
             }
-            &syn::NestedMeta::Meta(syn::Meta::NameValue(syn::MetaNameValue {
-                ref ident,
+            syn::NestedMeta::Meta(syn::Meta::NameValue(syn::MetaNameValue {
+                ref path,
                 ref lit,
                 ..
-            })) => match lit {
-                &syn::Lit::Str(ref value) => Some(Cfg::Named(format!("{}", ident), value.value())),
-                _ => None,
+            })) => match *lit {
+                syn::Lit::Str(ref value) => Cfg::Named(
+                    format!("{}", path.segments.first().unwrap().ident),
+                    value.value(),
+                ),
+                _ => return None,
             },
-            &syn::NestedMeta::Meta(syn::Meta::List(syn::MetaList {
-                ref ident,
+            syn::NestedMeta::Meta(syn::Meta::List(syn::MetaList {
+                ref path,
                 ref nested,
                 ..
             })) => {
-                if ident == "any" {
-                    if let Some(configs) = Cfg::load_list(nested.iter()) {
-                        Some(Cfg::Any(configs))
-                    } else {
-                        None
-                    }
-                } else if ident == "all" {
-                    if let Some(configs) = Cfg::load_list(nested.iter()) {
-                        Some(Cfg::All(configs))
-                    } else {
-                        None
-                    }
-                } else if ident == "not" {
+                if path.is_ident("any") {
+                    Cfg::Any(Cfg::load_list(nested.iter())?)
+                } else if path.is_ident("all") {
+                    Cfg::All(Cfg::load_list(nested.iter())?)
+                } else if path.is_ident("not") {
                     if nested.len() != 1 {
                         return None;
                     }
 
-                    if let Some(config) = Cfg::load_single(&nested[0]) {
-                        Some(Cfg::Not(Box::new(config)))
-                    } else {
-                        None
-                    }
+                    Cfg::Not(Box::new(Cfg::load_single(&nested[0])?))
                 } else {
-                    None
+                    return None;
                 }
             }
-            _ => None,
-        }
+            _ => return None,
+        })
     }
 
     fn load_list<'a, I: Iterator<Item = &'a syn::NestedMeta>>(attrs: I) -> Option<Vec<Cfg>> {
         let mut configs = Vec::new();
 
         for attr in attrs {
-            if let Some(config) = Cfg::load_single(attr) {
-                configs.push(config);
-            } else {
-                return None;
-            }
+            configs.push(Cfg::load_single(attr)?);
         }
 
         if configs.is_empty() {
@@ -195,41 +198,19 @@ impl Cfg {
 }
 
 pub trait ToCondition: Sized {
-    type Output;
-
-    fn to_condition(self, config: &Config) -> Option<Self::Output>;
+    fn to_condition(&self, config: &Config) -> Option<Condition>;
 }
 
-impl<'a> ToCondition for &'a Option<Cfg> {
-    type Output = Condition;
-
-    fn to_condition(self, config: &Config) -> Option<Self::Output> {
-        self.clone().and_then(|cfg| cfg.to_condition(config))
+impl<'a> ToCondition for Option<Cfg> {
+    fn to_condition(&self, config: &Config) -> Option<Condition> {
+        self.as_ref()?.to_condition(config)
     }
 }
 
-impl ToCondition for Option<Cfg> {
-    type Output = Condition;
-
-    fn to_condition(self, config: &Config) -> Option<Self::Output> {
-        self.and_then(|cfg| cfg.to_condition(config))
-    }
-}
-
-impl<'a> ToCondition for &'a Cfg {
-    type Output = Condition;
-
-    fn to_condition(self, config: &Config) -> Option<Self::Output> {
-        self.clone().to_condition(config)
-    }
-}
-
-impl ToCondition for Cfg {
-    type Output = Condition;
-
-    fn to_condition(self, config: &Config) -> Option<Self::Output> {
-        match self {
-            Cfg::Boolean(cfg_name) => {
+impl<'a> ToCondition for Cfg {
+    fn to_condition(&self, config: &Config) -> Option<Condition> {
+        match *self {
+            Cfg::Boolean(ref cfg_name) => {
                 let define = config
                     .defines
                     .iter()
@@ -239,12 +220,12 @@ impl ToCondition for Cfg {
                 } else {
                     warn!(
                         "Missing `[defines]` entry for `{}` in cbindgen config.",
-                        Cfg::Boolean(cfg_name)
+                        self,
                     );
                     None
                 }
             }
-            Cfg::Named(cfg_name, cfg_value) => {
+            Cfg::Named(ref cfg_name, ref cfg_value) => {
                 let define = config.defines.iter().find(|(key, ..)| {
                     DefineKey::Named(&cfg_name, &cfg_value) == DefineKey::load(key)
                 });
@@ -253,14 +234,14 @@ impl ToCondition for Cfg {
                 } else {
                     warn!(
                         "Missing `[defines]` entry for `{}` in cbindgen config.",
-                        Cfg::Named(cfg_name, cfg_value)
+                        self,
                     );
                     None
                 }
             }
-            Cfg::Any(children) => {
+            Cfg::Any(ref children) => {
                 let conditions: Vec<_> = children
-                    .into_iter()
+                    .iter()
                     .filter_map(|x| x.to_condition(config))
                     .collect();
                 match conditions.len() {
@@ -269,9 +250,9 @@ impl ToCondition for Cfg {
                     _ => Some(Condition::Any(conditions)),
                 }
             }
-            Cfg::All(children) => {
+            Cfg::All(ref children) => {
                 let cfgs: Vec<_> = children
-                    .into_iter()
+                    .iter()
                     .filter_map(|x| x.to_condition(config))
                     .collect();
                 match cfgs.len() {
@@ -280,7 +261,7 @@ impl ToCondition for Cfg {
                     _ => Some(Condition::All(cfgs)),
                 }
             }
-            Cfg::Not(child) => child
+            Cfg::Not(ref child) => child
                 .to_condition(config)
                 .map(|cfg| Condition::Not(Box::new(cfg))),
         }
@@ -297,13 +278,13 @@ pub enum Condition {
 
 impl Condition {
     fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
-        match self {
-            &Condition::Define(ref define) => {
+        match *self {
+            Condition::Define(ref define) => {
                 out.write("defined(");
                 write!(out, "{}", define);
                 out.write(")");
             }
-            &Condition::Any(ref conditions) => {
+            Condition::Any(ref conditions) => {
                 out.write("(");
                 for (i, condition) in conditions.iter().enumerate() {
                     if i != 0 {
@@ -313,7 +294,7 @@ impl Condition {
                 }
                 out.write(")");
             }
-            &Condition::All(ref conditions) => {
+            Condition::All(ref conditions) => {
                 out.write("(");
                 for (i, condition) in conditions.iter().enumerate() {
                     if i != 0 {
@@ -323,7 +304,7 @@ impl Condition {
                 }
                 out.write(")");
             }
-            &Condition::Not(ref condition) => {
+            Condition::Not(ref condition) => {
                 out.write("!");
                 condition.write(config, out);
             }
@@ -338,9 +319,11 @@ pub trait ConditionWrite {
 
 impl ConditionWrite for Option<Condition> {
     fn write_before<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
-        if let &Some(ref cfg) = self {
+        if let Some(ref cfg) = *self {
+            out.push_set_spaces(0);
             out.write("#if ");
             cfg.write(config, out);
+            out.pop_set_spaces();
             out.new_line();
         }
     }
@@ -348,7 +331,9 @@ impl ConditionWrite for Option<Condition> {
     fn write_after<F: Write>(&self, _config: &Config, out: &mut SourceWriter<F>) {
         if self.is_some() {
             out.new_line();
+            out.push_set_spaces(0);
             out.write("#endif");
+            out.pop_set_spaces();
         }
     }
 }
