@@ -8,16 +8,16 @@ use std::io::Write;
 
 use syn::{self, UnOp};
 
-use bindgen::config::{Config, Language};
-use bindgen::declarationtyperesolver::DeclarationTypeResolver;
-use bindgen::dependencies::Dependencies;
-use bindgen::ir::{
+use crate::bindgen::config::{Config, Language};
+use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
+use crate::bindgen::dependencies::Dependencies;
+use crate::bindgen::ir::{
     AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, Item, ItemContainer, Path,
     Struct, ToCondition, Type,
 };
-use bindgen::library::Library;
-use bindgen::writer::{Source, SourceWriter};
-use bindgen::Bindings;
+use crate::bindgen::library::Library;
+use crate::bindgen::writer::{Source, SourceWriter};
+use crate::bindgen::Bindings;
 
 #[derive(Debug, Clone)]
 pub enum Literal {
@@ -160,7 +160,7 @@ impl Literal {
                     syn::Lit::Byte(ref value) => Ok(Literal::Expr(format!("{}", value.value()))),
                     syn::Lit::Char(ref value) => Ok(Literal::Expr(match value.value() as u32 {
                         0..=255 => format!("'{}'", value.value().escape_default()),
-                        other_code => format!(r"L'\u{:X}'", other_code),
+                        other_code => format!(r"L'\U{:08X}'", other_code),
                     })),
                     syn::Lit::Int(ref value) => {
                         Ok(Literal::Expr(value.base10_digits().to_string()))
@@ -456,7 +456,7 @@ impl Constant {
             && config.constant.allow_static_const
             && !associated_to_transparent;
 
-        let condition = (&self.cfg).to_condition(config);
+        let condition = self.cfg.to_condition(config);
         condition.write_before(config, out);
 
         let name = if in_body {
