@@ -6,19 +6,19 @@ use std::io::Write;
 
 use syn;
 
-use bindgen::config::{Config, Language};
-use bindgen::declarationtyperesolver::DeclarationTypeResolver;
-use bindgen::dependencies::Dependencies;
-use bindgen::ir::{
+use crate::bindgen::config::{Config, Language};
+use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
+use crate::bindgen::dependencies::Dependencies;
+use crate::bindgen::ir::{
     AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, GenericPath, Item,
     ItemContainer, Path, Repr, ReprStyle, ReprType, Struct, ToCondition, Type,
 };
-use bindgen::library::Library;
-use bindgen::mangle;
-use bindgen::monomorph::Monomorphs;
-use bindgen::rename::{IdentifierType, RenameRule};
-use bindgen::utilities::find_first_some;
-use bindgen::writer::{ListType, Source, SourceWriter};
+use crate::bindgen::library::Library;
+use crate::bindgen::mangle;
+use crate::bindgen::monomorph::Monomorphs;
+use crate::bindgen::rename::{IdentifierType, RenameRule};
+use crate::bindgen::utilities::find_first_some;
+use crate::bindgen::writer::{ListType, Source, SourceWriter};
 
 #[derive(Debug, Clone)]
 pub struct EnumVariant {
@@ -26,6 +26,7 @@ pub struct EnumVariant {
     pub export_name: String,
     pub discriminant: Option<i64>,
     pub body: Option<(String, Struct)>,
+    pub cfg: Option<Cfg>,
     pub documentation: Documentation,
 }
 
@@ -48,11 +49,12 @@ fn value_from_expr(val: &syn::Expr) -> Option<i64> {
 }
 
 impl EnumVariant {
-    pub fn load(
+    fn load(
         is_tagged: bool,
         variant: &syn::Variant,
         generic_params: GenericParams,
         mod_cfg: Option<&Cfg>,
+        self_path: &Path,
     ) -> Result<Self, String> {
         let discriminant = match variant.discriminant {
             Some((_, ref expr)) => match value_from_expr(expr) {
@@ -65,6 +67,7 @@ impl EnumVariant {
         fn parse_fields(
             is_tagged: bool,
             fields: &syn::punctuated::Punctuated<syn::Field, syn::token::Comma>,
+            self_path: &Path,
         ) -> Result<Vec<(String, Type, Documentation)>, String> {
             let mut res = Vec::new();
 
@@ -77,7 +80,8 @@ impl EnumVariant {
             }
 
             for (i, field) in fields.iter().enumerate() {
-                if let Some(ty) = Type::load(&field.ty)? {
+                if let Some(mut ty) = Type::load(&field.ty)? {
+                    ty.replace_self_with(self_path);
                     res.push((
                         match field.ident {
                             Some(ref ident) => ident.to_string(),
@@ -92,6 +96,7 @@ impl EnumVariant {
             Ok(res)
         }
 
+        let variant_cfg = Cfg::append(mod_cfg, Cfg::load(&variant.attrs));
         let body = match variant.fields {
             syn::Fields::Unit => None,
             syn::Fields::Named(ref fields) => {
@@ -99,13 +104,13 @@ impl EnumVariant {
                 Some(Struct::new(
                     path,
                     generic_params,
-                    parse_fields(is_tagged, &fields.named)?,
+                    parse_fields(is_tagged, &fields.named, self_path)?,
                     is_tagged,
                     true,
                     None,
                     false,
                     false,
-                    Cfg::append(mod_cfg, Cfg::load(&variant.attrs)),
+                    None,
                     AnnotationSet::load(&variant.attrs)?,
                     Documentation::none(),
                 ))
@@ -115,13 +120,13 @@ impl EnumVariant {
                 Some(Struct::new(
                     path,
                     generic_params,
-                    parse_fields(is_tagged, &fields.unnamed)?,
+                    parse_fields(is_tagged, &fields.unnamed, self_path)?,
                     is_tagged,
                     true,
                     None,
                     false,
                     true,
-                    Cfg::append(mod_cfg, Cfg::load(&variant.attrs)),
+                    None,
                     AnnotationSet::load(&variant.attrs)?,
                     Documentation::none(),
                 ))
@@ -140,6 +145,7 @@ impl EnumVariant {
                     body,
                 )
             }),
+            variant_cfg,
             Documentation::load(&variant.attrs),
         ))
     }
@@ -148,6 +154,7 @@ impl EnumVariant {
         name: String,
         discriminant: Option<i64>,
         body: Option<(String, Struct)>,
+        cfg: Option<Cfg>,
         documentation: Documentation,
     ) -> Self {
         let export_name = name.clone();
@@ -156,6 +163,7 @@ impl EnumVariant {
             export_name,
             discriminant,
             body,
+            cfg,
             documentation,
         }
     }
@@ -179,6 +187,7 @@ impl EnumVariant {
             self.body
                 .as_ref()
                 .map(|&(ref name, ref ty)| (name.clone(), ty.specialize(generic_values, mappings))),
+            self.cfg.clone(),
             self.documentation.clone(),
         )
     }
@@ -198,12 +207,15 @@ impl EnumVariant {
 
 impl Source for EnumVariant {
     fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
+        let condition = self.cfg.to_condition(config);
+        condition.write_before(config, out);
         self.documentation.write(config, out);
         write!(out, "{}", self.export_name);
         if let Some(discriminant) = self.discriminant {
             write!(out, " = {}", discriminant);
         }
         out.write(",");
+        condition.write_after(config, out);
     }
 }
 
@@ -250,7 +262,11 @@ impl Enum {
         }
     }
 
-    pub fn load(item: &syn::ItemEnum, mod_cfg: Option<&Cfg>) -> Result<Enum, String> {
+    pub fn load(
+        item: &syn::ItemEnum,
+        mod_cfg: Option<&Cfg>,
+        config: &Config,
+    ) -> Result<Enum, String> {
         let repr = Repr::load(&item.attrs)?;
         if repr.style == ReprStyle::Rust && repr.ty.is_none() {
             return Err("Enum is not marked with a valid #[repr(prim)] or #[repr(C)].".to_owned());
@@ -260,6 +276,7 @@ impl Enum {
             return Err("Enum is marked with #[repr(align(...))] or #[repr(packed)].".to_owned());
         }
 
+        let path = Path::new(item.ident.to_string());
         let generic_params = GenericParams::new(&item.generics);
 
         let mut variants = Vec::new();
@@ -271,6 +288,7 @@ impl Enum {
                 variant,
                 generic_params.clone(),
                 mod_cfg,
+                &path,
             )?;
             is_tagged = is_tagged || variant.body.is_some();
             variants.push(variant);
@@ -280,16 +298,32 @@ impl Enum {
 
         if let Some(names) = annotations.list("enum-trailing-values") {
             for name in names {
-                variants.push(EnumVariant::new(name, None, None, Documentation::none()));
+                variants.push(EnumVariant::new(
+                    name,
+                    None,
+                    None,
+                    None,
+                    Documentation::none(),
+                ));
             }
         }
 
-        let path = Path::new(item.ident.to_string());
+        if config.enumeration.add_sentinel(&annotations) {
+            variants.push(EnumVariant::new(
+                "Sentinel".to_owned(),
+                None,
+                None,
+                None,
+                Documentation::simple(" Must be last for serialization purposes"),
+            ));
+        }
+
         let tag = if is_tagged {
             Some("Tag".to_string())
         } else {
             None
         };
+
         Ok(Enum::new(
             path,
             generic_params,
@@ -425,6 +459,7 @@ impl Item for Enum {
                                 body.1.clone(),
                             )
                         }),
+                        variant.cfg.clone(),
                         variant.documentation.clone(),
                     )
                 })
@@ -494,17 +529,18 @@ impl Source for Enum {
     fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
         let size = self.repr.ty.map(|ty| match ty {
             ReprType::USize => "uintptr_t",
+            ReprType::U64 => "uint64_t",
             ReprType::U32 => "uint32_t",
             ReprType::U16 => "uint16_t",
             ReprType::U8 => "uint8_t",
             ReprType::ISize => "intptr_t",
+            ReprType::I64 => "int64_t",
             ReprType::I32 => "int32_t",
             ReprType::I16 => "int16_t",
             ReprType::I8 => "int8_t",
         });
 
-        let condition = (&self.cfg).to_condition(config);
-
+        let condition = self.cfg.to_condition(config);
         condition.write_before(config, out);
 
         self.documentation.write(config, out);
@@ -525,6 +561,14 @@ impl Source for Enum {
 
             write!(out, " {}", self.export_name());
             out.open_brace();
+
+            // Emit the pre_body section, if relevant
+            // Only do this here if we're writing C++, since the struct that wraps everything is starting here.
+            // If we're writing C, we aren't wrapping the enum and variant structs definitions, so the actual enum struct willstart down below
+            if let Some(body) = config.export.pre_body(&self.path) {
+                out.write_raw_block(body);
+                out.new_line();
+            }
         }
 
         let enum_name = if let Some(ref tag) = self.tag {
@@ -557,7 +601,11 @@ impl Source for Enum {
                 }
             }
         } else {
-            out.write("enum class");
+            if config.enumeration.enum_class(&self.annotations) {
+                out.write("enum class");
+            } else {
+                out.write("enum");
+            }
 
             if self.annotations.must_use {
                 if let Some(ref anno) = config.enumeration.must_use {
@@ -577,11 +625,6 @@ impl Source for Enum {
             }
             variant.write(config, out);
         }
-        if config.enumeration.add_sentinel(&self.annotations) {
-            out.new_line();
-            out.new_line();
-            out.write("Sentinel /* this must be last for serialization purposes. */");
-        }
 
         if config.language == Language::C && size.is_none() && config.style.generate_typedef() {
             out.close_brace(false);
@@ -615,8 +658,10 @@ impl Source for Enum {
                 if let Some((_, ref body)) = variant.body {
                     out.new_line();
                     out.new_line();
-
+                    let condition = variant.cfg.to_condition(config);
+                    condition.write_before(config, out);
                     body.write(config, out);
+                    condition.write_after(config, out);
                 }
             }
 
@@ -636,6 +681,14 @@ impl Source for Enum {
                 }
 
                 out.open_brace();
+
+                // Emit the pre_body section, if relevant
+                // Only do this if we're writing C, since the struct is starting right here.
+                // For C++, the struct wraps all of the above variant structs too, and we write the pre_body section at the begining of that
+                if let Some(body) = config.export.pre_body(&self.path) {
+                    out.write_raw_block(body);
+                    out.new_line();
+                }
             }
 
             // C++ allows accessing only common initial sequence of union
@@ -647,7 +700,8 @@ impl Source for Enum {
                 out.open_brace();
             }
 
-            if config.language == Language::C && !config.style.generate_typedef() {
+            if config.language == Language::C && !size.is_some() && !config.style.generate_typedef()
+            {
                 out.write("enum ");
             }
 
@@ -664,19 +718,26 @@ impl Source for Enum {
                 out.open_brace();
             }
 
-            for (i, &(ref field_name, ref body)) in self
-                .variants
-                .iter()
-                .filter_map(|variant| variant.body.as_ref())
-                .enumerate()
             {
-                if i != 0 {
-                    out.new_line();
-                }
-                if config.style.generate_typedef() {
-                    write!(out, "{} {};", body.export_name(), field_name);
-                } else {
-                    write!(out, "struct {} {};", body.export_name(), field_name);
+                let mut first = true;
+                for variant in &self.variants {
+                    let (field_name, body) = match variant.body {
+                        Some((ref field_name, ref body)) => (field_name, body),
+                        None => continue,
+                    };
+
+                    if !first {
+                        out.new_line();
+                    }
+                    first = false;
+                    let condition = variant.cfg.to_condition(config);
+                    condition.write_before(config, out);
+                    if config.style.generate_typedef() {
+                        write!(out, "{} {};", body.export_name(), field_name);
+                    } else {
+                        write!(out, "struct {} {};", body.export_name(), field_name);
+                    }
+                    condition.write_after(config, out);
                 }
             }
 
@@ -688,11 +749,16 @@ impl Source for Enum {
 
             // Emit convenience methods
             let derive_helper_methods = config.enumeration.derive_helper_methods(&self.annotations);
+            let derive_const_casts = config.enumeration.derive_const_casts(&self.annotations);
+            let derive_mut_casts = config.enumeration.derive_mut_casts(&self.annotations);
             if config.language == Language::Cxx && derive_helper_methods {
                 for variant in &self.variants {
                     out.new_line();
                     out.new_line();
 
+                    let condition = variant.cfg.to_condition(config);
+                    condition.write_before(config, out);
+
                     let arg_renamer = |name: &str| {
                         config
                             .function
@@ -758,9 +824,7 @@ impl Source for Enum {
                     out.new_line();
                     write!(out, "return result;");
                     out.close_brace(false);
-                }
 
-                for variant in &self.variants {
                     out.new_line();
                     out.new_line();
 
@@ -769,36 +833,27 @@ impl Source for Enum {
                     out.open_brace();
                     write!(out, "return tag == {}::{};", enum_name, variant.export_name);
                     out.close_brace(false);
-                }
-            }
 
-            let derive_const_casts = config.enumeration.derive_const_casts(&self.annotations);
-            let derive_mut_casts = config.enumeration.derive_mut_casts(&self.annotations);
-            if config.language == Language::Cxx
-                && derive_helper_methods
-                && (derive_const_casts || derive_mut_casts)
-            {
-                let assert_name = match config.enumeration.cast_assert_name {
-                    Some(ref n) => &**n,
-                    None => "assert",
-                };
-
-                for variant in &self.variants {
-                    let (member_name, body) = match variant.body {
-                        Some((ref member_name, ref body)) => (member_name, body),
-                        None => continue,
+                    let assert_name = match config.enumeration.cast_assert_name {
+                        Some(ref n) => &**n,
+                        None => "assert",
                     };
 
-                    let field_count = body.fields.len() - skip_fields;
-                    if field_count == 0 {
-                        continue;
-                    }
-
-                    let dig = field_count == 1 && body.tuple_struct;
                     let mut derive_casts = |const_casts: bool| {
+                        let (member_name, body) = match variant.body {
+                            Some((ref member_name, ref body)) => (member_name, body),
+                            None => return,
+                        };
+
+                        let field_count = body.fields.len() - skip_fields;
+                        if field_count == 0 {
+                            return;
+                        }
+
                         out.new_line();
                         out.new_line();
 
+                        let dig = field_count == 1 && body.tuple_struct;
                         if dig {
                             let field = body.fields.iter().skip(skip_fields).next().unwrap();
                             let return_type = field.1.clone();
@@ -836,6 +891,8 @@ impl Source for Enum {
                     if derive_mut_casts {
                         derive_casts(false)
                     }
+
+                    condition.write_after(config, out);
                 }
             }
 
@@ -866,6 +923,8 @@ impl Source for Enum {
                 out.open_brace();
                 for variant in &self.variants {
                     if let Some((ref variant_name, _)) = variant.body {
+                        let condition = variant.cfg.to_condition(config);
+                        condition.write_before(config, out);
                         write!(
                             out,
                             "case {}::{}: return {} == {}.{};",
@@ -875,6 +934,7 @@ impl Source for Enum {
                             other,
                             variant_name
                         );
+                        condition.write_after(config, out);
                         out.new_line();
                     }
                 }
@@ -926,6 +986,8 @@ impl Source for Enum {
                 out.open_brace();
                 for variant in &self.variants {
                     if let Some((ref variant_name, ref item)) = variant.body {
+                        let condition = variant.cfg.to_condition(config);
+                        condition.write_before(config, out);
                         write!(
                             out,
                             "case {}::{}: {}.~{}(); break;",
@@ -934,6 +996,7 @@ impl Source for Enum {
                             variant_name,
                             item.export_name(),
                         );
+                        condition.write_after(config, out);
                         out.new_line();
                     }
                 }
@@ -961,6 +1024,8 @@ impl Source for Enum {
                 out.open_brace();
                 for variant in &self.variants {
                     if let Some((ref variant_name, ref item)) = variant.body {
+                        let condition = variant.cfg.to_condition(config);
+                        condition.write_before(config, out);
                         write!(
                             out,
                             "case {}::{}: ::new (&{}) ({})({}.{}); break;",
@@ -971,6 +1036,7 @@ impl Source for Enum {
                             other,
                             variant_name,
                         );
+                        condition.write_after(config, out);
                         out.new_line();
                     }
                 }
@@ -1002,7 +1068,9 @@ impl Source for Enum {
                 }
             }
 
-            if let Some(body) = config.export.extra_body(&self.path) {
+            // Emit the post_body section, if relevant
+            if let Some(body) = config.export.post_body(&self.path) {
+                out.new_line();
                 out.write_raw_block(body);
             }
 
@@ -1013,7 +1081,6 @@ impl Source for Enum {
                 out.close_brace(true);
             }
         }
-
         condition.write_after(config, out);
     }
 }
