@@ -6,20 +6,20 @@ use std::io::Write;
 
 use syn;
 
-use bindgen::config::{Config, Language, LayoutConfig};
-use bindgen::declarationtyperesolver::DeclarationTypeResolver;
-use bindgen::dependencies::Dependencies;
-use bindgen::ir::SynFieldHelpers;
-use bindgen::ir::{
+use crate::bindgen::config::{Config, Language, LayoutConfig};
+use crate::bindgen::declarationtyperesolver::DeclarationTypeResolver;
+use crate::bindgen::dependencies::Dependencies;
+use crate::bindgen::ir::SynFieldHelpers;
+use crate::bindgen::ir::{
     AnnotationSet, Cfg, ConditionWrite, Documentation, GenericParams, Item, ItemContainer, Path,
     Repr, ReprAlign, ReprStyle, ToCondition, Type,
 };
-use bindgen::library::Library;
-use bindgen::mangle;
-use bindgen::monomorph::Monomorphs;
-use bindgen::rename::{IdentifierType, RenameRule};
-use bindgen::utilities::{find_first_some, IterHelpers};
-use bindgen::writer::{ListType, Source, SourceWriter};
+use crate::bindgen::library::Library;
+use crate::bindgen::mangle;
+use crate::bindgen::monomorph::Monomorphs;
+use crate::bindgen::rename::{IdentifierType, RenameRule};
+use crate::bindgen::utilities::{find_first_some, IterHelpers};
+use crate::bindgen::writer::{ListType, Source, SourceWriter};
 
 #[derive(Debug, Clone)]
 pub struct Union {
@@ -50,17 +50,19 @@ impl Union {
             layout_config.ensure_safe_to_represent(&align)?;
         }
 
+        let path = Path::new(item.ident.to_string());
+
         let (fields, tuple_union) = {
             let out = item
                 .fields
                 .named
                 .iter()
-                .try_skip_map(|x| x.as_ident_and_type())?;
+                .try_skip_map(|x| x.as_ident_and_type(&path))?;
             (out, false)
         };
 
         Ok(Union::new(
-            Path::new(item.ident.to_string()),
+            path,
             GenericParams::new(&item.generics),
             fields,
             repr.align,
@@ -258,7 +260,7 @@ impl Item for Union {
 
 impl Source for Union {
     fn write<F: Write>(&self, config: &Config, out: &mut SourceWriter<F>) {
-        let condition = (&self.cfg).to_condition(config);
+        let condition = self.cfg.to_condition(config);
         condition.write_before(config, out);
 
         self.documentation.write(config, out);
@@ -299,6 +301,12 @@ impl Source for Union {
 
         out.open_brace();
 
+        // Emit the pre_body section, if relevant
+        if let Some(body) = config.export.pre_body(&self.path) {
+            out.write_raw_block(body);
+            out.new_line();
+        }
+
         if config.documentation {
             out.write_vertical_source_list(&self.fields, ListType::Cap(";"));
         } else {
@@ -310,7 +318,9 @@ impl Source for Union {
             out.write_vertical_source_list(&vec[..], ListType::Cap(";"));
         }
 
-        if let Some(body) = config.export.extra_body(&self.path) {
+        // Emit the post_body section, if relevant
+        if let Some(body) = config.export.post_body(&self.path) {
+            out.new_line();
             out.write_raw_block(body);
         }
 
